/* -*- Mode: C; tab-width: 3; indent-tabs-mode: t; c-basic-offset: 3 -*- */
/*
  data_interpretor.c
  heraia - an hexadecimal file editor and analyser based on ghex
 
  (C) Copyright 2005 Olivier Delhomme
  e-mail : heraia@delhomme.org
  URL    : http://heraia.tuxfamily.org
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or  (at your option) 
  any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY;  without even the implied warranty of
  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

#include <gtk/gtk.h>
#include <gtkhex/gtkhex.h>

#include "data_interpretor.h"
#include "decode.h"
#include "ghex_heraia_interface.h"

/* Begining of data interpretor things  */
/*
  Here we want to know where we are in the file (from the beginning)
*/
static void DI_print_offset( GtkWidget *hexwidget, 
							 Data_Window_Struct *DW )
{	
	guint pos;      /* position of the cursor in the file */
	gchar *text;    /* interpreted text to be printed     */
	GtkHex *gh;

	gh = GTK_HEX (hexwidget);

	pos = gtk_hex_get_cursor (gh);

	/* test if were are in the file that we are curently editing */
	if((pos >= 0) && (pos <= gh->document->file_size))
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "offset = %u", pos);
			gtk_statusbar_pop (GTK_STATUSBAR(DW->window_statusbar), 
									 DW->statusbar_context );
			DW->statusbar_context =
				gtk_statusbar_get_context_id (GTK_STATUSBAR(DW->window_statusbar),
														text);
			gtk_statusbar_push (GTK_STATUSBAR(DW->window_statusbar), 
									  DW->statusbar_context, text);
			g_free (text);
		}
	else
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "offset is unknown");
			gtk_statusbar_pop (GTK_STATUSBAR(DW->window_statusbar), 
									 DW->statusbar_context );
			DW->statusbar_context =
				gtk_statusbar_get_context_id (GTK_STATUSBAR(DW->window_statusbar),
														text);
			gtk_statusbar_push (GTK_STATUSBAR(DW->window_statusbar), 
									  DW->statusbar_context, text);
			g_free (text);
		}
}

/*
    Here we do interpret an unsigned byte in decimal format
*/
static void interpret_unsigned_byte( GtkWidget *hexwidget, 
									 Data_Window_Struct *DW )
{	
	guint pos;      /* position of the cursor in the file */
	gchar *text;    /* interpreted text to be printed     */
	guchar c;       /* the character under the cursor     */
	GtkHex *gh;

	gh = GTK_HEX (hexwidget);

	pos = gtk_hex_get_cursor (gh);

	/* test if were are in the file that we are curently editing */
	if((pos >= 0) && (pos <= gh->document->file_size))
		{
			c = gtk_hex_get_byte(gh, pos);
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%u", c);
			gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_byte), text);
			g_free (text);
		}
	else
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Not a number");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_byte), text);
			g_free (text);
		}
}

/*
    Here we do interpret a signed byte in decimal format
*/
static void interpret_signed_byte( GtkWidget *hexwidget, 
								   Data_Window_Struct *DW )
{	
	guint pos;      /* position of the cursor in the file */
	gchar *text;    /* interpreted text to be printed     */
	gchar c;        /* the character under the cursor     */
	GtkHex *gh;

	gh = GTK_HEX (hexwidget);

	pos = gtk_hex_get_cursor (gh);

	/* test if were are in the file that we are curently editing */
	if((pos >= 0) && (pos <= gh->document->file_size))
		{
			c = gtk_hex_get_byte(gh, pos);
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%d", c);
			gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_byte), text);
			g_free (text);
		}
	else
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Not a number");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_byte), text);
			g_free (text);
		}
}

/*
    Here we do interpret a 16 bits number in decimal format
	 is_signed == TRUE interpets as a signed 16 bits number
	 is_signed == FALSE interprets as an unsigned 16 bits number
*/
static void interpret_16bits_number( gboolean is_signed, GtkWidget *hexwidget, 
												 Data_Window_Struct *DW )
{	
	gchar *text; /* interpreted text to be printed     */
	guchar *c;   /* the characters under the cursor and the next one */
	gint result; /* used to test different results of function calls */
	GtkHex *gh;  /* the GtkHex widget we want to communicate with    */

	gh = GTK_HEX (hexwidget);

	c = (guchar *) g_malloc0 (2*sizeof(guchar));
	result = ghex_memcpy(gh, gtk_hex_get_cursor (gh), 2, c);

	if (result == TRUE )
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			if (is_signed == TRUE)
				result = decode_16bits_signed(c, text);
			else
				result = decode_16bits_unsigned(c, text);

			if (result == TRUE)
				{
					if (is_signed == TRUE)
						gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_16bits), text);
					else
						gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_16bits), text);
				}
			g_free (text);
		}
	else
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Not a 16 bits number");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_16bits), text);
			gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_16bits), text);
			g_free (text);
		}
}

/*
    Here we do interpret a 32 bits number in decimal format
	 is_signed == TRUE interpets as a signed 32 bits number
	 is_signed == FALSE interprets as an unsigned 32 bits number
*/
static void interpret_32bits_number( gboolean is_signed, GtkWidget *hexwidget, 
												 Data_Window_Struct *DW )
{	
	gchar *text; /* interpreted text to be printed     */
	guchar *c;   /* the characters under the cursor and the next one */
	gint result; /* used to test different results of function calls */
	GtkHex *gh;  /* the GtkHex widget we want to communicate with    */

	gh = GTK_HEX (hexwidget);

	c = (guchar *) g_malloc0 (4*sizeof(guchar));
	result = ghex_memcpy(gh, gtk_hex_get_cursor (gh), 4, c);

	if (result == TRUE )
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			if (is_signed == TRUE)
				result = decode_32bits_signed(c, text);
			else
				result = decode_32bits_unsigned(c, text);

			if (result == TRUE)
				{
					if (is_signed == TRUE)
						gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_32bits), text);
					else
						gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_32bits), text);
				}
			g_free (text);
		}
	else
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Not a 32 bits number");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_32bits), text);
			gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_32bits), text);
			g_free (text);
		}
}

/*
    Here we do interpret a 64 bits number in decimal format
	 is_signed == TRUE interpets as a signed 64 bits number
	 is_signed == FALSE interprets as an unsigned 64 bits number
*/
static void interpret_64bits_number( gboolean is_signed, GtkWidget *hexwidget, 
												 Data_Window_Struct *DW )
{	
	gchar *text; /* interpreted text to be printed     */
	guchar *c;   /* the characters under the cursor and the next one */
	gint result; /* used to test different results of function calls */
	GtkHex *gh;  /* the GtkHex widget we want to communicate with    */

	gh = GTK_HEX (hexwidget);

	c = (guchar *) g_malloc0 (8*sizeof(guchar));
	result = ghex_memcpy(gh, gtk_hex_get_cursor (gh), 8, c);

	if (result == TRUE )
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			if (is_signed == TRUE)
				result = decode_64bits_signed(c, text);
			else
				result = decode_64bits_unsigned(c, text);

			if (result == TRUE)
				{
					if (is_signed == TRUE)
						gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_64bits), text);
					else
						gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_64bits), text);
				}
			g_free (text);
		}
	else
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Not a 64 bits number");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_64bits), text);
			gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_64bits), text);
			g_free (text);
		}
}


/*
    Here we do interpret a dos date (a 4 bytes data)
	 We can not have any date struct type because our interprets
    might differ from any real date (when interpreted data does
    not represent a dos date).
*/
static void interpret_dos_date( GtkWidget *hexwidget, 
								   Data_Window_Struct *DW )
{	
	gchar *text; /* interpreted text to be printed                   */
	guchar *c;   /* the characters under the cursor and the next one */
	GtkHex *gh;  /* the GtkHex widget we want to communicate with    */
	gint result; /* used to test different results of function calls */

	gh = GTK_HEX (hexwidget);
	
	c = (guchar *) g_malloc0 (4*sizeof(guchar));
	
	result = ghex_memcpy(gh, gtk_hex_get_cursor (gh), 4, c);

	if (result == TRUE )
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			result = decode_dos_date(c, text);
			if (result == TRUE)
				gtk_entry_set_text (GTK_ENTRY(DW->entry_dos_date), text);
			g_free (text);
		}
	
	if (result == FALSE )
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Not a dos date");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_dos_date), text);
			g_free (text);
		}

	g_free (c);
}

/*
    Here we do interpret a filetime date (a 8 bytes data)
	 We can not have any date struct type because our interprets
    might differ from any real date (when interpreted data does
    not represent a filetime date).
*/
static void interpret_filetime_date( GtkWidget *hexwidget, 
												 Data_Window_Struct *DW )
{	
	gchar *text; /* interpreted text to be printed                   */
	guchar *c;   /* the characters under the cursor and the next one */
	GtkHex *gh;  /* the GtkHex widget we want to communicate with    */
	gint result; /* used to test different results of function calls */
	date_and_time *mydate;

	gh = GTK_HEX (hexwidget);
	
	c = (guchar *) g_malloc0 (8*sizeof(guchar));		
	mydate = (date_and_time *) g_malloc0 (sizeof(*mydate));
	text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
	
	result = ghex_memcpy(gh, gtk_hex_get_cursor (gh), 8, c);
	
	if (result == TRUE )
		{
			result = decode_filetime_date(c, mydate);
			if (result == TRUE)
				{
					sprintf(text, "%02u/%02u/%u - %02u:%02u:%02u",
							  mydate->day, mydate->month, mydate->year, 
							  mydate->hour, mydate->minutes, mydate->seconds);
					gtk_entry_set_text (GTK_ENTRY(DW->entry_filetime_date), text);
				}		
		}
	
	if (result == FALSE )
		{
			sprintf(text, "%s", "Not a filetime date");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_filetime_date), text);
		}

	g_free (text);
	g_free(mydate);
	g_free (c);
}

/*
    Here we do interpret a C date (a 4 bytes data)
	 We can not have any date struct type because our interprets
    might differ from any real date (when interpreted data does
    not represent a C date).
*/
static void interpret_C_date( GtkWidget *hexwidget, 
										Data_Window_Struct *DW )
{	
	gchar *text; /* interpreted text to be printed                   */
	guchar *c;   /* the characters under the cursor and the next one */
	GtkHex *gh;  /* the GtkHex widget we want to communicate with    */
	gint result; /* used to test different results of function calls */
	date_and_time *mydate; /* date resulting of interpretation       */


	gh = GTK_HEX (hexwidget);
	
	c = (guchar *) g_malloc0 (4*sizeof(guchar));
	mydate = (date_and_time *) g_malloc0 (sizeof(*mydate));
	text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));

	result = ghex_memcpy(gh, gtk_hex_get_cursor (gh), 4, c);

	if (result == TRUE )
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			result = decode_C_date(c, mydate);

			if (result == TRUE)
				{
					sprintf(text, "%02u/%02u/%u - %02u:%02u:%02u", 
							  mydate->day, mydate->month, mydate->year, 
							  mydate->hour, mydate->minutes, mydate->seconds);
					gtk_entry_set_text (GTK_ENTRY(DW->entry_C_date), text);
				}
		}
	
	if (result == FALSE )
		{
			sprintf(text, "%s", "Not a C date");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_C_date), text);
		}
	
	g_free (text);
	g_free (mydate);
	g_free (c);
}


/* End of data interpretor things  */


/* Begining of Widget related stuff  */

static gboolean delete_dt_window_event( GtkWidget *widget, 
						      GdkEvent  *event,
						      gpointer   data )
{
  gtk_widget_hide (widget);
  return TRUE;
}

static void destroy_dt_window( GtkWidget *widget,
					     gpointer   data )
{
  gtk_widget_hide (widget);
}

/*
  function called when the cursor changes its position
*/
void refresh_data_window( GtkWidget *hexwidget, Data_Window_Struct *DW )
{
	DI_print_offset(hexwidget, DW);
	if (DW->display_8bits == TRUE)
		{
			interpret_unsigned_byte(hexwidget, DW);
			interpret_signed_byte(hexwidget, DW);
		}

	if (DW->display_16bits == TRUE)
		{
			interpret_16bits_number(FALSE, hexwidget, DW);
			interpret_16bits_number(TRUE, hexwidget, DW);
		}

	if (DW->display_32bits == TRUE)
		{
			interpret_32bits_number(FALSE, hexwidget, DW);
			interpret_32bits_number(TRUE, hexwidget, DW);
		}

	if (DW->display_64bits == TRUE)
		{
			interpret_64bits_number(FALSE, hexwidget, DW);
			interpret_64bits_number(TRUE, hexwidget, DW);
		}

	if (DW->display_dos_date == TRUE)
		interpret_dos_date(hexwidget, DW);

	if (DW->display_filetime_date == TRUE)
		interpret_filetime_date(hexwidget, DW);

	if (DW->display_C_date == TRUE)
		interpret_C_date(hexwidget, DW);

	gtk_widget_show_all (DW->window);
}

/* display functions below */

static void display_8bits( GtkWidget *hexwidget, Data_Window_Struct *DW, gushort line )
{
	/* First line is number 0 */
	/* label */
	DW->label_8bits = gtk_label_new ("8 bits");
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->label_8bits),
						  0, 1, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE); 
	
	/* unsigned byte entry */
	DW->entry_unsigned_byte = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->entry_unsigned_byte), 
						  1, 2, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);
 
	/* signed byte entry */
	DW->entry_signed_byte = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->entry_signed_byte), 
						  2, 3, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);

	interpret_unsigned_byte(hexwidget, DW);
	interpret_signed_byte(hexwidget, DW);
		
}

static void display_16bits( GtkWidget *hexwidget, Data_Window_Struct *DW, 
							gushort line )
{
	/* First line is number 0 */
	/* label */
	DW->label_16bits = gtk_label_new ("16 bits");
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->label_16bits),
						  0, 1, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);

	/* unsigned 16 bits entry*/
	DW->entry_unsigned_16bits = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->entry_unsigned_16bits), 
						  1, 2, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);
	
	/* signed 16 bits entry */
	DW->entry_signed_16bits = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->entry_signed_16bits), 
						  2, 3, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
					 H_DI_H_SPACE, H_DI_V_SPACE);

	interpret_16bits_number(FALSE, hexwidget, DW);
	interpret_16bits_number(TRUE, hexwidget, DW);
}

static void display_32bits( GtkWidget *hexwidget, Data_Window_Struct *DW, 
							gushort line )
{
	/* First line is number 0 */
	/* label */
	DW->label_32bits = gtk_label_new ("32 bits");
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->label_32bits), 
						  0, 1, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);

	/* unsigned 32 bits entry*/
	DW->entry_unsigned_32bits = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY); 
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->entry_unsigned_32bits), 
						  1, 2, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);
  
	/* signed 32 bits entry */
	DW->entry_signed_32bits = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->entry_signed_32bits), 
						  2, 3, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
					 H_DI_H_SPACE, H_DI_V_SPACE);

	interpret_32bits_number(FALSE, hexwidget, DW);
	interpret_32bits_number(TRUE, hexwidget, DW);

}

static void display_64bits( GtkWidget *hexwidget, Data_Window_Struct *DW, 
							gushort line )
{
	/* First line is number 0 */
	/* label */
	DW->label_64bits = gtk_label_new ("64 bits");
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->label_64bits), 
						  0, 1, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);

	/* unsigned 64 bits entry*/
	DW->entry_unsigned_64bits = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY); 
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->entry_unsigned_64bits), 
						  1, 2, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);

	/* signed 64 bits entry */
	DW->entry_signed_64bits = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->entry_signed_64bits), 
						  2, 3, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);

	interpret_64bits_number(FALSE, hexwidget, DW);
	interpret_64bits_number(TRUE, hexwidget, DW);
}

static void display_dos_date( GtkWidget *hexwidget, Data_Window_Struct *DW, 
							  gushort line )
{
	/* dos date */
	/* First line is number 0 */
	/* label */
	DW->label_dos_date = gtk_label_new ("dos date");
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->label_dos_date),
						  0, 1, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);
	/* entry */
	DW->entry_dos_date = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->entry_dos_date), 
						  1, 2, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);

	interpret_dos_date(hexwidget, DW);
}

static void display_filetime_date( GtkWidget *hexwidget, Data_Window_Struct *DW, 
									 gushort line )
{
	/* filetime date */
	/* First line is number 0 */
	/* label */
	DW->label_filetime_date = gtk_label_new ("filetime date");
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->label_filetime_date), 
						  0, 1, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);
	/* entry */
	DW->entry_filetime_date = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->entry_filetime_date), 
						  1, 2, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);

	interpret_filetime_date(hexwidget, DW);
}

static void display_C_date( GtkWidget *hexwidget, Data_Window_Struct *DW, 
									 gushort line )
{
	/* C date */
	/* First line is number 0 */
	/* label */
	DW->label_C_date = gtk_label_new ("C date");
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->label_C_date), 
						  0, 1, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);
	/* entry */
	DW->entry_C_date = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->entry_C_date), 
						  1, 2, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);

	interpret_C_date(hexwidget, DW);
}

static void display_labels( GtkWidget *hexwidget, Data_Window_Struct *DW, 
									 gushort line )
{
	/* unsigned byte */
	DW->label_type = gtk_label_new ("type");
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->label_type), 
						  0, 1, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE); 

	/* unsigned byte */
	DW->label_unsigned = gtk_label_new ("unsigned");
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->label_unsigned), 
						  1, 2, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE); 

	/* signed byte */
	DW->label_signed = gtk_label_new ("signed");
	gtk_table_attach(GTK_TABLE(DW->window_table), 
						  GTK_WIDGET(DW->label_signed), 
						  2, 3, line, line+1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
						  H_DI_H_SPACE, H_DI_V_SPACE);
}

void display_di_window( GtkWidget *hexwidget, Data_Window_Struct *DW ) 
{
	gushort line = 0;
	if (DW->display_8bits == TRUE || DW->display_16bits == TRUE ||
		 DW->display_32bits == TRUE || DW->display_64bits == TRUE )
		display_labels(hexwidget, DW, line++);

	if (DW->display_8bits == TRUE)
		display_8bits(hexwidget, DW, line++);
	
	if (DW->display_16bits == TRUE)
		display_16bits(hexwidget, DW, line++);

	if (DW->display_32bits == TRUE)
		display_32bits(hexwidget, DW, line++);

	if (DW->display_64bits == TRUE)
		display_64bits(hexwidget, DW, line++);

	if (DW->display_dos_date == TRUE)
		display_dos_date(hexwidget, DW, line++);

	if (DW->display_filetime_date == TRUE)
		display_filetime_date(hexwidget, DW, line++);

	if (DW->display_C_date == TRUE)
		display_C_date(hexwidget, DW, line++);
}


static void init_display_defaults(Data_Window_Struct *DW)
{
	/* Here we could load display settings from a configuration file */

	DW->display_8bits = TRUE;
	DW->display_16bits = TRUE;
	DW->display_32bits = TRUE;
	DW->display_64bits = TRUE;
	DW->display_dos_date = TRUE;
	DW->display_filetime_date = TRUE;
	DW->display_C_date = TRUE;
}

static void display_statusbar( GtkWidget *hexwidget, Data_Window_Struct *DW )
{


	DW->statusbar_context = 
		gtk_statusbar_get_context_id (GTK_STATUSBAR(DW->window_statusbar), "");

	DI_print_offset(hexwidget, DW);
}



/*
  Initialise the Data_Window_Struct whose aim is to 
  display the data interpreted. Following interpret
  is done automatically :
  8, 16, 32 and 64 bits, dos date and filetime date !
*/
 void data_interpret( GtkWidget *hexwidget, Data_Window_Struct *DW )
{

	DW->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	  
	DW->window_vbox = gtk_vbox_new (FALSE, H_DI_V_SPACE);
	gtk_container_add (GTK_CONTAINER(DW->window), DW->window_vbox);
  	
	DW->window_table = gtk_table_new (H_DI_LINES, H_DI_COLUMNS, TRUE);
	gtk_container_add (GTK_CONTAINER(DW->window_vbox), DW->window_table);

	DW->window_statusbar = gtk_statusbar_new () ;
	gtk_container_add (GTK_CONTAINER(DW->window_vbox), DW->window_statusbar);
	
	init_display_defaults(DW);
	display_statusbar(hexwidget, DW);
	display_di_window(hexwidget, DW);

	g_signal_connect (G_OBJECT (DW->window), "delete_event", 
					  G_CALLBACK (delete_dt_window_event), NULL);
	g_signal_connect (G_OBJECT (DW->window), "destroy", 
					  G_CALLBACK (destroy_dt_window), NULL);

	/* Get this out of here if we want MDI driven window */
	g_signal_connect (G_OBJECT (hexwidget), "cursor_moved",
					  G_CALLBACK (refresh_data_window), DW);

	gtk_widget_show_all (DW->window);
}
/* End of Widget related stuff  */
